@@ -37,6 +37,8 @@ static char sccsid[] = "@(#)term.c	5.1 (Berkeley) 12/22/91";
 
 #include <sys/types.h>
 #include <errno.h>
+#include <termios.h>
+#include <sys/ioctl.h>
 /* #include <ttyent.h> */
 #include <unistd.h>
 #include <stdio.h>
@@ -45,8 +47,9 @@ static char sccsid[] = "@(#)term.c	5.1 (Berkeley) 12/22/91";
 #include "extern.h"
 #include "map.h"
 #include "misc.h"
+#include "ttytype.h"
 
-char    tbuf[1024];      		/* Termcap entry. */
+char    tbuf[TERMCAP_BUFSIZE];      		/* Termcap entry. */
 
 /*
 char	*askuser __P((char *));
@@ -86,7 +89,10 @@ get_termcap_entry(userarg, tcapbufp)
 	char *userarg, **tcapbufp;
 {
 	int rval;
-	char *ttype;
+	char *ttype, *base, *t, *ttypath;
+#ifdef TIOCGWINSZ
+	struct winsize win;
+#endif
 
 	if (userarg) {
 		ttype = userarg;
@@ -98,23 +104,36 @@ get_termcap_entry(userarg, tcapbufp)
 		goto map;
 
 	/* Try ttyname(3); check for dialup or other mapping. */
-/*
-	if (ttypath = ttyname(STDERR_FILENO)) {
-		if (base = rindex(ttypath, '/'))
+	if ((ttypath = ttyname(STDERR_FILENO)) != NULL) {
+		if ((base = rindex(ttypath, '/')))
 			++base;
 		else
 			base = ttypath;
+#ifdef DEBUG
+		fprintf (stderr, "Running on %s\n", base);
+#endif
+#ifdef HAVE_GETTTYNAM
 		if ((t = getttynam(base))) {
 			ttype = t->ty_type;
 			goto map;
 		}
+#else
+		if ((t = get_ttytype(base)) != NULL) {
+			ttype = t;
+			goto map;
+		}
+#endif
 	}
-*/
+
 	/* If still undefined, use "unknown". */
 	ttype = "unknown";
 
 map:	ttype = mapped(ttype);
 
+#ifdef DEBUG
+	fprintf (stderr, "Default terminal type %s\n", ttype);
+#endif
+
 	/*
 	 * Remove TERMCAP from the environment so we get a real entry from
 	 * /etc/termcap.  This prevents us from being fooled by out of date
@@ -122,6 +141,24 @@ map:	ttype = mapped(ttype);
 	 */
 found:	unsetenv("TERMCAP");
 
+	/*
+	 * Also zero out tty driver entries for window size if they are
+	 * to be forced from the termcap file
+	 */
+#ifdef TIOCGWINSZ
+	if (dosetwin) {
+		(void)ioctl(STDERR_FILENO, TIOCGWINSZ, &win);
+		olines = win.ws_row;
+		ocolumns = win.ws_col;
+#ifdef DEBUG
+		fprintf (stderr, "osize: %d rows, %d columns\n", olines, ocolumns);
+#endif
+		win.ws_row = 0;
+		win.ws_col = 0;
+		(void)ioctl(STDERR_FILENO, TIOCSWINSZ, &win);
+	}
+#endif
+
 	/*
 	 * ttype now contains a pointer to the type of the terminal.
 	 * If the first character is '?', ask the user.
